{%- extends "layout.html" %}

{% block extrahead %}
  {% if pagename == 'facebook-example' %}
    <link rel="stylesheet" type="text/css" href="{{ pathto('_static/facebook-example.css', 1) }}" />
    <script src="https://cdnjs.cloudflare.com/ajax/libs/d3/3.5.5/d3.min.js"></script>
    <script src="{{ pathto('_static/facebook-example.js', 1) }}"></script>
  {% endif %}
{% endblock %}


{% block body %}
  {{ body }}
{% endblock %}
