# {{ ansible_managed }}
description "girder worker"

start on started mountall
stop on shutdown

# Automatically Respawn:
respawn
respawn limit 10 10

setuid "{{ girder_worker_user }}"
setgid "{{ girder_worker_user }}"

script
  exec "{{ girder_worker_executable }}"
end script

pre-stop script
  exec "{{ celery_executable }} multi stopwait worker"
end script
